/*
 * Copyright 2016-2023 NXP
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of NXP Semiconductor, Inc. nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * @file    SPI2_Template.c
 * @brief   Application entry point.
 */
#include <stdio.h>
#include "board.h"
#include "peripherals.h"
#include "pin_mux.h"
#include "clock_config.h"
#include "MK66F18.h"
#include "fsl_debug_console.h"
/* TODO: insert other include files here. */
#define TRANSFER_SIZE 64U

dspi_slave_handle_t g_s_handle;
volatile bool isTransferCompleted = true;
uint8_t slaveRxData[TRANSFER_SIZE] = {0U};
uint8_t slaveTxData[TRANSFER_SIZE] = {0U};
uint8_t ADC_prepared[2];
uint8_t UART_Buffer[16];
/* TODO: insert other definitions and declarations here. */
typedef enum {
    expect_header,
    recieve_UART,
    send_ADC,
}state_machine;

state_machine decode(uint8_t header){
    switch(header){
    case 0xCC:
        return recieve_UART;
    case 0x55:
        return send_ADC;
    default:
        return expect_header;
    }
}

void DSPI_SlaveUserCallback(SPI_Type *base, dspi_slave_handle_t *handle, status_t status, void *userData)
{
    static state_machine state = expect_header;
    static bool header = true;
    dspi_transfer_t* transfer = userData;

    // TODO: add counter (see slides)
    if (status == kStatus_Success)
    {
        if(header){
            switch(state = decode(transfer->rxData[0])){
                case expect_header:
                    // TODO: unexpected header occurred, wait again
                    header = true;
                    break;
                case recieve_UART:
                    // TODO: prepare transfer expecting 16 bytes of UART data
                    header = false;
                    break;
                case send_ADC:
                    // TODO: prepare transfer that sends 12 ADC bits
                    header = false;
                    break;
            }
        }else{
            if(recieve_UART == state){
                // TODO: handle received UART data
            }

            // TODO: transfer finished, except new header
            header = true;
        }

        DSPI_SlaveTransferNonBlocking(SPI0_PERIPHERAL, &g_s_handle, transfer);
    }

    if (status == kStatus_DSPI_Error)
    {
        PRINTF("Error occurred in this transfer. \r\n\r\n");
    }
}
/*
 * @brief   Application entry point.
 */
int main(void) {

    /* Init board hardware. */
    BOARD_InitBootPins();
    BOARD_InitBootClocks();
    BOARD_InitBootPeripherals();
#ifndef BOARD_INIT_DEBUG_CONSOLE_PERIPHERAL
    /* Init FSL debug console. */
    BOARD_InitDebugConsole();
#endif

    PRINTF("Hello World\n");
    dspi_transfer_t slaveXfer;
    DSPI_SlaveTransferCreateHandle(SPI0_PERIPHERAL, &g_s_handle, DSPI_SlaveUserCallback, &slaveXfer);
    slaveTxData[0] = 'A';
    slaveTxData[1] = 'C';
    slaveTxData[2] = 'B';

    isTransferCompleted   = false;
    slaveXfer.txData      = slaveTxData;
    slaveXfer.rxData      = slaveRxData;
    slaveXfer.dataSize    = 1;
    slaveXfer.configFlags = kDSPI_SlaveCtar0;

    /* Slave start receive */
    DSPI_SlaveTransferNonBlocking(SPI0_PERIPHERAL, &g_s_handle, &slaveXfer);

    while(1) {
        asm("nop");
    }

    return 0;
}
